@@ -31,6 +31,7 @@ import tempfile
 
 from PyQt5.QtCore import QDir, QFile, QFileInfo, QIODevice
 
+from ..metadata import pyqt4_metadata, pyqt5_metadata
 from ..project import QrcDirectory, QrcFile
 from ..user_exception import UserException
 from ..version import PYQTDEPLOY_HEXVERSION
@@ -39,104 +40,43 @@ from ..version import PYQTDEPLOY_HEXVERSION
 class Builder():
     """ The builder for a project. """
 
-    # Map PyQt modules to the corresponding qmake QT or CONFIG values.  A
-    # module that doesn't need any of these values can be omitted
-    class _Qt5MetaData:
-        def __init__(self, qt=None, config=None, needs_suffix=False):
-            self.qt = qt
-            self.config = config
-            self.needs_suffix = needs_suffix
-
-    class _Qt4MetaData(_Qt5MetaData):
-        def __init__(self, qt=None, config=None, needs_suffix=True):
-            super().__init__(qt=qt, config=config, needs_suffix=needs_suffix)
-
-    _pyqt5_module_map = {
-        'QAxContainer':         _Qt5MetaData(qt=['axcontainer']),
-        'QtBluetooth':          _Qt5MetaData(qt=['bluetooth']),
-        'QtCore':               _Qt5MetaData(qt=['-gui']),
-        'QtDBus':               _Qt5MetaData(qt=['dbus', '-gui']),
-        'QtDesigner':           _Qt5MetaData(qt=['designer']),
-        'QtHelp':               _Qt5MetaData(qt=['help']),
-        'QtMacExtras':          _Qt5MetaData(qt=['macextras']),
-        'QtMultimedia':         _Qt5MetaData(qt=['multimedia']),
-        'QtMultimediaWidgets':  _Qt5MetaData(
-                                        qt=['multimediawidgets',
-                                                'multimedia']),
-        'QtNetwork':            _Qt5MetaData(qt=['network', '-gui']),
-        'QtOpenGL':             _Qt5MetaData(qt=['opengl']),
-        'QtPositioning':        _Qt5MetaData(qt=['positioning']),
-        'QtPrintSupport':       _Qt5MetaData(qt=['printsupport']),
-        'QtQml':                _Qt5MetaData(qt=['qml']),
-        'QtQuick':              _Qt5MetaData(qt=['quick']),
-        'QtSensors':            _Qt5MetaData(qt=['sensors']),
-        'QtSerialPort':         _Qt5MetaData(qt=['serialport']),
-        'QtSql':                _Qt5MetaData(qt=['sql', 'widgets']),
-        'QtSvg':                _Qt5MetaData(qt=['svg']),
-        'QtTest':               _Qt5MetaData(qt=['testlib', 'widgets']),
-        'QtWebKit':             _Qt5MetaData(qt=['webkit', 'network']),
-        'QtWebKitWidgets':      _Qt5MetaData(qt=['webkitwidgets']),
-        'QtWidgets':            _Qt5MetaData(qt=['widgets']),
-        'QtWinExtras':          _Qt5MetaData(qt=['winextras', 'widgets']),
-        'QtX11Extras':          _Qt5MetaData(qt=['x11extras']),
-        'QtXmlPatterns':        _Qt5MetaData(
-                                        qt=['xmlpatterns', '-gui', 'network']),
-
-        'QtChart':              _Qt5MetaData(config=['qtcommercialchart']),
-        'QtDataVisualization':  _Qt5MetaData(qt=['datavisualization']),
-        'Qsci':                 _Qt5MetaData(config=['qscintilla2']),
-    }
-
-    _pyqt4_module_map = {
-        'QAxContainer':         _Qt4MetaData(config=['qaxcontainer']),
-        'QtCore':               _Qt4MetaData(qt=['-gui']),
-        'QtDBus':               _Qt4MetaData(qt=['dbus', '-gui']),
-        'QtDeclarative':        _Qt4MetaData(qt=['declarative', 'network']),
-        'QtDesigner':           _Qt4MetaData(config=['designer']),
-        'QtHelp':               _Qt4MetaData(config=['help']),
-        'QtMultimedia':         _Qt4MetaData(qt=['multimedia']),
-        'QtNetwork':            _Qt4MetaData(qt=['network', '-gui']),
-        'QtOpenGL':             _Qt4MetaData(qt=['opengl']),
-        'QtScript':             _Qt4MetaData(qt=['script', '-gui']),
-        'QtScriptTools':        _Qt4MetaData(qt=['scripttools', 'script']),
-        'QtSql':                _Qt4MetaData(qt=['sql']),
-        'QtSvg':                _Qt4MetaData(qt=['svg']),
-        'QtTest':               _Qt4MetaData(qt=['testlib']),
-        'QtWebKit':             _Qt4MetaData(qt=['webkit', 'network']),
-        'QtXml':                _Qt4MetaData(qt=['xml', '-gui']),
-        'QtXmlPatterns':        _Qt4MetaData(
-                                        qt=['xmlpatterns', '-gui', 'network']),
-        'phonon':               _Qt4MetaData(qt=['phonon']),
-
-        'QtChart':              _Qt4MetaData(config=['qtcommercialchart'],
-                                        needs_suffix=False),
-        'Qsci':                 _Qt4MetaData(config=['qscintilla2'],
-                                        needs_suffix=False),
-    }
-
-    def __init__(self, project, verbose=False):
+    def __init__(self, project):
         """ Initialise the builder for a project. """
 
         super().__init__()
 
         self._project = project
-        self._verbose = verbose
 
-    def build(self, build_dir):
+        self.quiet = False
+        self.verbose = False
+
+    def build(self, build_dir=None, clean=False, console=False):
         """ Build the project in a given directory.  Raise a UserException if
         there is an error.
         """
 
         project = self._project
 
+        if build_dir is None:
+            build_dir = project.build_dir
+            if build_dir == '':
+                build_dir = '.'
+
+            build_dir = project.absolute_path(build_dir)
+
+        if clean:
+            self._progress("Cleaning {0}.".format(build_dir))
+            shutil.rmtree(build_dir, ignore_errors=True)
+
         self._create_directory(build_dir)
 
         freeze = self._copy_lib_file('freeze.py')
 
-        self._write_qmake(build_dir, freeze)
+        self._write_qmake(build_dir, console, freeze)
 
         resources_dir = os.path.join(build_dir, 'resources')
         self._create_directory(resources_dir)
+        stdlib_dir = project.absolute_path(project.python_target_stdlib_dir)
 
         for resource in self.resources():
             if resource == '':
@@ -145,8 +85,7 @@ class Builder():
                         project.absolute_path(package.name), freeze)
             elif resource == 'stdlib':
                 self._write_resource(resources_dir, resource,
-                        project.stdlib_package,
-                        os.path.join(project.python_target_stdlib_dir, ''),
+                        project.stdlib_package, os.path.join(stdlib_dir, ''),
                         freeze)
             else:
                 # Add the PyQt package to a temporary copy of the site-packages
@@ -163,16 +102,13 @@ class Builder():
                     # Add uic if requested.
                     if 'uic' in project.pyqt_modules:
                         self._add_uic_dir(pyqt_pkg_dir,
-                                os.path.join(
-                                        self._project.python_target_stdlib_dir,
-                                        'site-packages', pyqt_dir),
+                                os.path.join(stdlib_dir, 'site-packages',
+                                        pyqt_dir),
                                 'uic', [])
 
                 self._write_resource(resources_dir, resource,
                         site_packages_package,
-                        os.path.join(project.python_target_stdlib_dir,
-                                'site-packages', ''),
-                        freeze)
+                        os.path.join(stdlib_dir, 'site-packages', ''), freeze)
 
         os.remove(freeze)
 
@@ -198,13 +134,12 @@ class Builder():
                 self._add_uic_dir(dir_pkg, pyqt_dir, content, dir_stack)
                 dir_stack.pop()
 
-    def _write_qmake(self, build_dir, freeze):
+    def _write_qmake(self, build_dir, console, freeze):
         """ Create the .pro file for qmake. """
 
         project = self._project
 
-        app_name = os.path.basename(project.application_script)
-        app_name, _ = os.path.splitext(app_name)
+        app_name = project.application_basename()
 
         f = self._create_file(build_dir, app_name + '.pro')
 
@@ -221,25 +156,25 @@ class Builder():
         for pyqt_m in project.pyqt_modules:
             needs_gui = True
 
-            metadata = self._get_metadata(pyqt_m)
+            metadata = self._get_module_metadata(pyqt_m)
 
-            if metadata.qt is not None:
-                for qt in metadata.qt:
-                    if qt == '-gui':
-                        needs_gui = False
-                    elif qt not in qmake_qt:
-                        qmake_qt.append(qt)
+            for qt in metadata.qt:
+                if qt == '-gui':
+                    needs_gui = False
+                elif qt not in qmake_qt:
+                    qmake_qt.append(qt)
 
-            if metadata.config is not None:
-                for config in metadata.config:
-                    if config not in qmake_config:
-                        qmake_config.append(config)
+            for config in metadata.config:
+                if config not in qmake_config:
+                    qmake_config.append(config)
 
             if needs_gui:
                 no_gui = False
 
-        if no_gui:
+        if no_gui or console:
             f.write('CONFIG += console\n')
+
+        if no_gui:
             f.write('QT -= gui\n')
 
         if len(qmake_qt) != 0:
@@ -260,14 +195,17 @@ class Builder():
 
         if len(project.pyqt_modules) > 0:
             sitepackages = QDir.fromNativeSeparators(
-                    project.absolute_path(
-                            project.python_target_stdlib_dir)) + '/site-packages'
+                    os.path.join(
+                            project.absolute_path(
+                                    project.python_target_stdlib_dir),
+                            'site-packages'))
+
             pyqt_version = 'PyQt5' if project.application_is_pyqt5 else 'PyQt4'
 
-            for pyqt in project.pyqt_modules:
+            for pyqt in self._get_all_pyqt_modules():
                 if pyqt != 'uic':
                     lib_name = pyqt
-                    if self._get_metadata(pyqt).needs_suffix:
+                    if self._get_module_metadata(pyqt).needs_suffix:
                         # Qt4's qmake thinks -lQtCore etc. always refer to the
                         # Qt libraries so PyQt4 creates static libraries with a
                         # suffix.
@@ -305,11 +243,9 @@ class Builder():
             f.write('INCLUDEPATH += {0}\n'.format(self._quote(inc_dir)))
 
         if project.python_target_library != '':
-            lib_dir = os.path.dirname(project.python_target_library)
-            lib_dir = QDir.fromNativeSeparators(project.absolute_path(lib_dir))
-
-            lib, _ = os.path.splitext(
-                    os.path.basename(project.python_target_library))
+            lib_path = project.absolute_path(project.python_target_library)
+            lib_dir = QDir.fromNativeSeparators(os.path.dirname(lib_path))
+            lib, _ = os.path.splitext(os.path.basename(lib_path))
 
             if lib.startswith('lib'):
                 lib = lib[3:]
@@ -320,7 +256,7 @@ class Builder():
         platforms_f = QFile(self._lib_filename('platforms.pro'))
 
         if not platforms_f.open(QIODevice.ReadOnly|QIODevice.Text):
-            raise UserException(
+            self._error(
                     "Unable to open file {0}.".format(platforms_f.fileName()),
                     platforms_f.errorString())
 
@@ -358,7 +294,7 @@ class Builder():
 
         major, _ = project.python_target_version
         if major is None:
-            raise UserException("Unable to determine target Python version")
+            self._error("Unable to determine target Python version")
 
         bootstrap = self._copy_lib_file(
                 'bootstrap_py3.py' if major == 3 else 'bootstrap_py2.py')
@@ -403,17 +339,38 @@ class Builder():
 
         return resources
 
-    def _get_metadata(self, name):
+    def _get_module_metadata(self, module_name):
         """ Get the metadata for a module. """
 
         if self._project.application_is_pyqt5:
-            module_map = self._pyqt5_module_map
-            default_metadata = self._Qt5MetaData()
+            metadata = pyqt5_metadata
         else:
-            module_map = self._pyqt4_module_map
-            default_metadata = self._Qt4MetaData()
+            metadata = pyqt4_metadata
 
-        return module_map.get(name, default_metadata)
+        return metadata[module_name]
+
+    def _get_all_pyqt_modules(self):
+        """ Return the list of all PyQt modules including dependencies. """
+
+        all_modules = []
+
+        for module_name in self._project.pyqt_modules:
+            self._get_module_dependencies(module_name, all_modules)
+
+            if module_name not in all_modules:
+                all_modules.append(module_name)
+
+        return all_modules
+
+    def _get_module_dependencies(self, module_name, all_modules):
+        """ Update a list of dependencies for a module. """
+
+        for dep in self._get_module_metadata(module_name).deps:
+            if dep not in all_modules:
+                all_modules.append(dep)
+
+            # Handle sub-dependencies.
+            self._get_module_dependencies(dep, all_modules)
 
     @staticmethod
     def _quote(name):
@@ -599,25 +556,45 @@ int main(int argc, char **argv)
     def _freeze(self, output, py_filename, freeze, name=None, as_data=False):
         """ Freeze a Python source file to a C header file or a data file. """
 
-        args = [self._project.python_host_interpreter, freeze]
+        argv = [os.path.expandvars(self._project.python_host_interpreter),
+            freeze]
         
         if name is not None:
-            args.append('--name')
-            args.append(name)
+            argv.append('--name')
+            argv.append(name)
 
-        args.append('--as-data' if as_data else '--as-c')
-        args.append(output)
-        args.append(py_filename)
+        argv.append('--as-data' if as_data else '--as-c')
+        argv.append(output)
+        argv.append(py_filename)
 
-        self._log("Running '{0}'".format(' '.join(args)))
+        self._progress("Freezing {0}".format(py_filename))
+
+        self.run(argv, "Unable to freeze {0}.".format(py_filename))
+
+    def run(self, argv, error_message, in_build_dir=False):
+        """ Execute a command and capture the output. """
+
+        if in_build_dir:
+            saved_cwd = os.getcwd()
+            build_dir = self._project.absolute_path(self._project.build_dir)
+            os.chdir(build_dir)
+            self._verbose(
+                    "{0} is now the current directory.".format(build_dir))
+        else:
+            saved_cwd = None
+
+        self._verbose("Running '{0}'".format(' '.join(argv)))
 
         try:
-            subprocess.check_output(args, stderr=subprocess.STDOUT,
+            subprocess.check_output(argv, stderr=subprocess.STDOUT,
                     universal_newlines=True)
         except subprocess.CalledProcessError as e:
-            self._log(e.output)
-            raise UserException("Unable to freeze {0}.".format(py_filename),
-                    e.output)
+            self._error(error_message, e.output)
+        finally:
+            if saved_cwd is not None:
+                os.chdir(saved_cwd)
+                self._verbose(
+                        "{0} is now the current directory.".format(saved_cwd))
 
     @staticmethod
     def _lib_filename(filename):
@@ -653,7 +630,7 @@ int main(int argc, char **argv)
         QFile.remove(d_filename)
 
         if not QFile.copy(s_filename, QDir.fromNativeSeparators(d_filename)):
-            raise UserException("Unable to copy file {0}.".format(filename))
+            self._error("Unable to copy file {0}.".format(filename))
 
         return d_filename
 
@@ -666,22 +643,52 @@ int main(int argc, char **argv)
         try:
             return open(pathname, 'wt')
         except Exception as e:
-            raise UserException("Unable to create file {0}.".format(pathname),
-                    str(e))
+            self._error("Unable to create file {0}.".format(pathname), str(e))
 
-    @staticmethod
-    def _create_directory(dir_name):
+    def _create_directory(self, dir_name):
         """ Create a directory which may already exist. """
 
+        self._verbose("Creating directory {0}.".format(dir_name))
+
         try:
             os.makedirs(dir_name, exist_ok=True)
         except Exception as e:
-            raise UserException(
+            self._error(
                     "Unable to create the '{0}' directory.".format(dir_name),
                     str(e))
 
-    def _log(self, message):
-        """ Log a message if requested. """
+    def _progress(self, text):
+        """ Display a progress message if requested. """
 
-        if self._verbose:
-            print(message)
+        if not self.quiet:
+            self.information(text)
+
+    def _verbose(self, text):
+        """ Display a verbose progress message if requested. """
+
+        if self.verbose:
+            self.information(text)
+
+    def _error(self, text, detail=''):
+        """ Handle an error.  This will raise an exception and not return. """
+
+        self.error(text)
+
+        if detail != '':
+            self.error(detail)
+
+        raise UserException(text, detail)
+
+    def information(self, text):
+        """ Handle a user information message (which will not have a trailing
+        newline).  This default implementation just sends it to stdout.
+        """
+
+        print(text)
+
+    def error(self, text):
+        """ Handle a user error message (which will not have a trailing
+        newline).  This default implementation just sends it to stderr.
+        """
+
+        print(text, file=sys.stderr)
