@@ -98,13 +98,15 @@ class _SitePackagesPackageEditor(QrcPackageEditor):
     def get_root_dir(self):
         """ Get the name of the Python site packages directory. """
 
-        stdlib_dir = self._project.python_target_stdlib_dir
+        project = self._project
+
+        stdlib_dir = project.absolute_path(project.python_target_stdlib_dir)
 
         if stdlib_dir == '':
             QMessageBox.warning(self, self._title,
                     "site-packages cannot be scanned because the directory "
                     "name of the standard library has not been set in the "
-                    "Python Configuration tab.")
+                    "Locations tab.")
             return ''
 
         return os.path.join(stdlib_dir, 'site-packages')
